package com.hzya.frame.plugin.mdmDistribute.plugin;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.mdm.mdmModule.dao.IMdmModuleDao;
import com.hzya.frame.mdm.mdmModule.entity.MdmModuleEntity;
import com.hzya.frame.mdm.mdmModuleDb.entity.MdmModuleDbEntity;
import com.hzya.frame.mdm.mdmModuleDbFileds.entity.MdmModuleDbFiledsEntity;
import com.hzya.frame.mdm.mdmModuleDbFiledsRule.dao.IMdmModuleDbFiledsRuleDao;
import com.hzya.frame.mdm.mdmModuleDbFiledsRule.entity.MdmModuleDbFiledsRuleEntity;
import com.hzya.frame.mdm.mdmModuleDistribute.dao.IMdmModuleDistributeDao;
import com.hzya.frame.mdm.mdmModuleDistribute.entity.MdmModuleDistributeEntity;
import com.hzya.frame.mdm.service.IMdmServiceCache;
import com.hzya.frame.sysnew.application.api.dao.ISysApplicationApiDao;
import com.hzya.frame.sysnew.application.api.entity.SysApplicationApiEntity;
import com.hzya.frame.sysnew.application.dao.ISysApplicationDao;
import com.hzya.frame.sysnew.application.entity.SysApplicationEntity;
import com.hzya.frame.sysnew.application.entity.SysExtensionApiEntity;
import com.hzya.frame.sysnew.application.script.dao.ISysApplicationScriptDao;
import com.hzya.frame.sysnew.application.script.entity.SysApplicationScriptEntity;
import com.hzya.frame.sysnew.grovy.service.IGroovyIntegrationService;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.service.IIntegrationTaskLivingDetailsService;
import com.hzya.frame.sysnew.messageManageLog.entity.SysMessageManageLogEntity;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import javax.annotation.Resource;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 主数据模版(MdmModule)表服务接口
 *
 * @author makejava
 * @since 2024-06-18 10:33:32
 */
public class MdmModulePluginInitializer extends PluginBaseEntity {
    Logger logger = LoggerFactory.getLogger(MdmModulePluginInitializer.class);
    @Resource
    private IIntegrationTaskLivingDetailsDao integrationTaskLivingDetailsDao;
    @Resource
    private IIntegrationTaskLivingDetailsService taskLivingDetailsService;
    @Resource
    private ISysApplicationApiDao sysApplicationApiDao;
    @Resource
    private ISysApplicationDao sysApplicationDao;
    @Resource
    private ISysApplicationScriptDao sysApplicationScriptDao;
    @Resource
    private IMdmModuleDistributeDao mdmModuleDistributeDao;
    @Resource
    private IGroovyIntegrationService groovyIntegrationService;
    @Resource
    private IMdmModuleDbFiledsRuleDao mdmModuleDbFiledsRuleDao;
    @Resource
    private IMdmModuleDao mdmModuleDao;
    @Resource
    private IMdmServiceCache mdmServiceCache;

    @Value("${ax.url}")
    private String url ;

    //多线程请求加锁 HttpRequest 构造方法是静态的
    private final Object lock = new Object();
    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "MdmModulePlugin";
    }

    @Override
    public String getPluginName() {
        return "MdmModulePlugin插件";
    }

    @Override
    public String getPluginLabel() {
        return "MdmModulePlugin";
    }

    @Override
    public String getPluginType() {
        return "1";
    }

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) {
        try{
            //重试方法
            if (null != requestJson) {
                //如果这个id不为空，说明是重试的
                String id = requestJson.getString("integration_task_living_details_id");
                //执行重新发送
                if (StrUtil.isNotEmpty(id)) {
                    //查询日志表
                    IntegrationTaskLivingDetailsEntity taskDetailEntity = taskLivingDetailsService.get(id);
                    if (null != taskDetailEntity && JSONUtil.isTypeJSON(taskDetailEntity.getRootAppPk())){
                        //拿到这张表的源系统ID
                        JSONObject jsonObject = JSONObject.parseObject(taskDetailEntity.getRootAppPk());
                        Long mdmCode = jsonObject.getLong("mdmCode");//主数据编码
                        String documentRule = jsonObject.getString("documentRule");//行数据的单据规则编码
                        String distributeId = jsonObject.getString("distributeId");//发送表id
                        String type = jsonObject.getString("type");//发送类型，1、新增2、修改3、删除
                        if(mdmCode == null || documentRule == null || distributeId == null || type == null
                            || "".equals(mdmCode) || "".equals(documentRule) || "".equals(distributeId) || "".equals(type)){
                            taskDetailEntity.setResult("系统保存参数错误");
                            taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("系统保存参数错误");
                        }
                        String apiId = null;
                        String scriptId = null;
                        //查找对应数据，组装数据发送

                        //获取主数据信息
                        MdmModuleEntity mdmModuleEntity = mdmServiceCache.getMdmModuleEntity(mdmCode);
                        if(mdmModuleEntity == null || mdmModuleEntity.getId() == null ){
                            taskDetailEntity.setResult("主数据不存在");
                            taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("主数据不存在");

                        }

                        //获取主数据db
                        MdmModuleDbEntity mdmModuleDbEntity = new MdmModuleDbEntity();
                        mdmModuleDbEntity.setMdmId(mdmModuleEntity.getId());
                        List<MdmModuleDbEntity> mdmModuleDbEntities = mdmServiceCache.queryMdmModuleDb(mdmModuleDbEntity);
                        if(mdmModuleDbEntities == null || mdmModuleDbEntities.size() == 0){
                            taskDetailEntity.setResult("主数据数据源不存在");
                            taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("主数据数据源不存在");

                        }

                        //获取分发表
                        //获取分发的接口
                        MdmModuleDistributeEntity mdmModuleDistributeEntity = mdmModuleDistributeDao.get(distributeId);
                        if(mdmModuleDistributeEntity == null || mdmModuleDistributeEntity.getId() == null ){
                            taskDetailEntity.setResult("分发不存在");
                            taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("分发不存在");

                        }
                        // 启用停用 0、停用
                        if("0".equals(mdmModuleDistributeEntity.getEnabledState())){
                            taskDetailEntity.setResult("分发已停用");
                            taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("分发已停用");

                        }
                        // 启用停用 0、停用
                        //if("1".equals(mdmModuleDistributeEntity.getEnabledType())){
                        //    taskDetailEntity.setResult("分发为手动下发");
                        //    taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                        //    return BaseResult.getFailureMessageEntity("分发为手动下发");
                        //}
                        //1、新增2、修改3、删除
                        if("1".equals(type)){
                            apiId = mdmModuleDistributeEntity.getAddApi();
                            scriptId = mdmModuleDistributeEntity.getAddScript();
                        }else if("2".equals(type)){
                            apiId = mdmModuleDistributeEntity.getUpdateApi();
                            scriptId = mdmModuleDistributeEntity.getUpdateScript();

                        }else if("3".equals(type)){
                            apiId = mdmModuleDistributeEntity.getDeleteApi();
                            scriptId = mdmModuleDistributeEntity.getDeleteScript();
                        }
                        if(apiId == null || "".equals(apiId)){
                            taskDetailEntity.setResult("分发接口未选择，请先选择");
                            taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("分发接口未选择，请先选择");

                        }
                        SysApplicationScriptEntity scriptEntity = null;
                        if(scriptId != null && !"".equals(scriptId)){
                            scriptEntity = sysApplicationScriptDao.get(scriptId);
                            if(scriptEntity == null || scriptEntity.getId() == null ){
                                taskDetailEntity.setResult("分发接口脚本不存在");
                                taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                                return BaseResult.getFailureMessageEntity("分发接口脚本不存在");
                            }
                        }
                        SysApplicationApiEntity apiEntity = sysApplicationApiDao.get(apiId);
                        if(apiEntity == null || apiEntity.getId() == null ){
                            taskDetailEntity.setResult("分发接口不存在");
                            taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("分发接口不存在");

                        }
                        SysApplicationEntity sysApplicationEntity = sysApplicationDao.get(apiEntity.getAppId());
                        if(sysApplicationEntity == null || sysApplicationEntity.getId() == null ){
                            taskDetailEntity.setResult("应用不存在");
                            taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("应用不存在");

                        }

                        //获取主数据db字段，以及字段的规则
                        MdmModuleDbFiledsEntity queryFild = new MdmModuleDbFiledsEntity();
                        queryFild.setMdmId(mdmModuleEntity.getId());
                        List<MdmModuleDbFiledsEntity> mdmModuleDbFiledsEntities = mdmServiceCache.queryMdmModuleDbFileds(queryFild);
                        MdmModuleDbFiledsRuleEntity queryFildRule = new MdmModuleDbFiledsRuleEntity();
                        queryFildRule.setMdmId(mdmModuleEntity.getId());
                        queryFildRule.setRuleCode("service");
                        List<MdmModuleDbFiledsRuleEntity> mdmModuleDbFiledsRuleEntities = mdmModuleDbFiledsRuleDao.queryBase(queryFildRule);
                        //获取主数据db对应的实体表的数据
                        JSONObject object = new JSONObject();
                        for (int i = 0; i < mdmModuleDbEntities.size(); i++) {
                            if("1".equals(mdmModuleDbEntities.get(i).getDbType())){
                                Map<String,String> map = new HashMap<>();
                                map.put("tableName",mdmModuleDbEntities.get(i).getDbName());
                                map.put("documentRule",documentRule);
                                object = mdmModuleDao.queryMdmST(map);
                                if(object.getString("id") == null || "".equals(object.getString("id")) ){
                                    taskDetailEntity.setResult("当前数据不存在");
                                    taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                                    return BaseResult.getFailureMessageEntity("当前数据不存在");
                                }
                                //查看主表是否有字段是关联的
                                if(mdmModuleDbFiledsRuleEntities != null && mdmModuleDbFiledsRuleEntities.size() > 0){
                                    for (int i1 = 0; i1 < mdmModuleDbFiledsRuleEntities.size(); i1++) {
                                        if(mdmModuleDbFiledsRuleEntities.get(i1).getDbId().equals(mdmModuleDbEntities.get(i).getId())){
                                            if(mdmModuleDbFiledsEntities != null && mdmModuleDbFiledsEntities.size() > 0){
                                                for (int i2 = 0; i2 < mdmModuleDbFiledsEntities.size(); i2++) {
                                                    if(mdmModuleDbFiledsRuleEntities.get(i1).getFiledId().equals(mdmModuleDbFiledsEntities.get(i2).getId())){
                                                        Map<String,String> mapDetail = new HashMap<>();
                                                        mapDetail.put("tableName",mdmModuleDbFiledsRuleEntities.get(i1).getRuleValue());
                                                        mapDetail.put("id",object.getString(mdmModuleDbFiledsEntities.get(i2).getEnName()));
                                                        JSONObject objectDetail = mdmModuleDao.queryMdmST(mapDetail);
                                                        object.put(mdmModuleDbFiledsEntities.get(i2).getEnName(),objectDetail);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        for (int i = 0; i < mdmModuleDbEntities.size(); i++) {
                            if("2".equals(mdmModuleDbEntities.get(i).getDbType())){
                                Map<String,String> map = new HashMap<>();
                                map.put("tableName",mdmModuleDbEntities.get(i).getDbName());
                                map.put("formmainId",object.getString("id"));
                                List<JSONObject> detail = mdmModuleDao.queryMdmSTDetail(map);
                                //查看主表是否有字段是关联的
                                if(detail != null && detail.size() > 0){
                                    if(mdmModuleDbFiledsRuleEntities != null && mdmModuleDbFiledsRuleEntities.size() > 0){
                                        for (int i1 = 0; i1 < mdmModuleDbFiledsRuleEntities.size(); i1++) {
                                            if(mdmModuleDbFiledsRuleEntities.get(i1).getDbId().equals(mdmModuleDbEntities.get(i).getId())){
                                                if(mdmModuleDbFiledsEntities != null && mdmModuleDbFiledsEntities.size() > 0){
                                                    for (int i2 = 0; i2 < mdmModuleDbFiledsEntities.size(); i2++) {
                                                        if(mdmModuleDbFiledsRuleEntities.get(i1).getFiledId().equals(mdmModuleDbFiledsEntities.get(i2).getId())){
                                                            for (int i3 = 0; i3 < detail.size(); i3++) {
                                                                Map<String,String> mapDetail = new HashMap<>();
                                                                mapDetail.put("tableName",mdmModuleDbFiledsRuleEntities.get(i1).getRuleValue());
                                                                mapDetail.put("id",detail.get(i3).getString(mdmModuleDbFiledsEntities.get(i2).getEnName()));
                                                                JSONObject objectDetail = mdmModuleDao.queryMdmST(mapDetail);
                                                                detail.get(i3).put(mdmModuleDbFiledsEntities.get(i2).getEnName(),objectDetail);
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                                object.put(mdmModuleDbEntities.get(i).getDbName(),detail);
                            }
                        }
                        //执行脚本
                        JSONObject groovy = new JSONObject();
                        JSONObject parameterJson = new JSONObject();
                        parameterJson.put("header",apiEntity.getHeaderIn());
                        parameterJson.put("body",apiEntity.getBodyIn());
                        parameterJson.put("query",apiEntity.getQueryIn());
                        parameterJson.put("data",object);
                        /**  head */
                        Map<String, String> header = null;
                        /**  query */
                        String querys = null;
                        /**  body */
                        String bodys = object.toJSONString();
                        if(scriptEntity != null && scriptEntity.getId() != null ){
                            groovy.put("code",scriptEntity.getScriptCode());
                            groovy.put("className",scriptEntity.getClassName());
                            groovy.put("name",scriptEntity.getScriptName());
                            groovy.put("methodStr",scriptEntity.getScriptData());
                            groovy.put("parameterJson",parameterJson);
                            JSONObject groovyStr = new JSONObject();
                            groovyStr.put("jsonStr",groovy);
                            try {
                                SysExtensionApiEntity jsonResultEntity =  groovyIntegrationService.groovyScriptExecution(groovy);
                                header = jsonResultEntity.getHeaders();
                                querys = jsonResultEntity.getQuerys();
                                bodys = jsonResultEntity.getBodys();
                            }catch (Exception e){
                                taskDetailEntity.setResult("分发脚本转换错误");
                                taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                                //return BaseResult.getFailureMessageEntity("分发脚本转换错误");
                            }
                        }
                        Map<String, String> headers = new HashMap<>();
                        headers.put("publicKey","ZJYAWb7lhAUTYqekPkU+uHJv1/ObJxb7dT7sD8HPRDGAgyhCe7eDIk+3zDUT+v578prj");
                        headers.put("secretKey","fviZnLBsQUAGF8w8FSOdJi7XlIm/XAZclMxRagDLfTyJFlvnIBF3w66Hrpfzs8cYj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=");
                        headers.put("appId",sysApplicationEntity.getAppId().toString());
                        headers.put("apiCode",apiEntity.getApiCode().toString());
                        if(header != null){
                            headers.putAll(header);
                        }
                        //组装数据发送
                        JsonResultEntity jsonResultEntity = sendData(apiEntity,headers,bodys,querys);
                        if(jsonResultEntity.isFlag()){
                            taskDetailEntity.setResult("发送成功");
                            taskLivingDetailsService.saveLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("发送成功");
                        }else {
                            taskDetailEntity.setResult("发送错误");
                            taskLivingDetailsService.updateLogFailToSuccess(taskDetailEntity);
                            return BaseResult.getFailureMessageEntity("发送错误");
                        }
                    }
                }else {
                    //执行正常下发


                }
            }
            logger.info("执行成功");
            return BaseResult.getSuccessMessageEntity("执行成功");
        } catch (Exception e) {
           logger.error("执行失败{}",e.getMessage());
        }
        return BaseResult.getSuccessMessageEntity("执行成功");
    }

    private JsonResultEntity sendData(SysApplicationApiEntity applicationApiEntity, Map<String, String> headers ,String bodys ,String querys) {
        StringBuffer urls = new StringBuffer(url);
        if (querys != null) {
            urls.append("?");
            urls.append(querys);
        }
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        // HttpClient
        CloseableHttpClient closeableHttpClient = httpClientBuilder.disableCookieManagement().build();
        HttpPost post = new HttpPost(urls.toString());
        CloseableHttpResponse response = null;

        RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(Integer.valueOf(applicationApiEntity.getTimeoutPeriod())).build();
        post.setConfig(requestConfig);//设置请求参数【超时时间】

        if (headers != null && headers.size() > 0) {
            for (String key : headers.keySet()) {
                post.setHeader(key, headers.get(key));
            }
        }
        StringBuilder body = new StringBuilder();
        boolean flag = true;
            try {
            if (bodys != null && !"".equals(bodys)) {
                ByteArrayEntity entity = new ByteArrayEntity(bodys.getBytes("UTF-8"));
                entity.setContentType("application/json");
                post.setEntity(entity);
            }
            response = closeableHttpClient.execute(post);

            HttpEntity entity = response.getEntity();
            synchronized (lock) {
                body.append(EntityUtils.toString(entity,"UTF-8"));
            }
            flag = true;
            logger.info("返回结果：" + body);
        } catch (Exception e) {
            logger.error("请求错误：" + e.getMessage());
            body.append(e.getMessage());
            flag = false;
        } finally {
            try {
                // 关闭响应对象
                if (response != null) {
                    response.close();
                }
                // 关闭响应对象
                if (closeableHttpClient != null) {
                    closeableHttpClient.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

        }
        if (flag) {
            if (JSONUtil.isTypeJSON(body.toString())) {
                JsonResultEntity jsonResultEntity = JSONObject.parseObject(body.toString(),JsonResultEntity.class);
                if(jsonResultEntity.isFlag()){
                    return BaseResult.getSuccessMessageEntity("转发成功", body);
                }else {
                    return BaseResult.getFailureMessageEntity("转发失败", body);
                }
            }else {
                return BaseResult.getFailureMessageEntity("转发失败", body);
            }
        } else {
            return BaseResult.getFailureMessageEntity("转发失败", body);
        }
    }
}
